/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/

// PhysBCUtil.cpp

//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// Dan Martin, Fri, Jan 14, 2000


/*****************/
/*****************/

#include "PhysBCUtil.H"
#include "PoissonBC.H"
#include "ExtrapBC.H"
#include "HOExtrapBC.H"
#include "ReflectBC.H"
#include "ReflectOddBC.H"
#include "scalarDirichletBC.H"
#include "viscousBC.H"
#include "basicVelBC.H"
#include "boxVelBC.H"
#include "edgeGhostBC.H"
#include "MayDay.H"

PhysBCUtil::PhysBCUtil()
{
  // initialize to bogus values
  for (int dir=0; dir<SpaceDim; dir++) {
    m_loBC[dir] = bogusBC;
    m_hiBC[dir] = bogusBC;
  }

  // now initialize BC's
  setBCs();

}


PhysBCUtil::~PhysBCUtil()
{
}

int
PhysBCUtil::loBC(int a_dir) const
{
  return m_loBC[a_dir];
}

int 
PhysBCUtil::hiBC(int a_dir) const
{
  return m_hiBC[a_dir];
}
  
int 
PhysBCUtil::getBC(int a_dir, const Side::LoHiSide& a_side) const 
{
  int bcVal;
  if (a_side == Side::Lo) {
    bcVal = loBC(a_dir);
  } else if (a_side == Side::Hi) {
    bcVal = hiBC(a_dir);
  } else {
    MayDay::Error("PhysBCUtil::getBC -- invalid side");
  }
  return bcVal;
}


PhysBCUtil&
PhysBCUtil::operator= (const PhysBCUtil& rhs)
{
  m_loBC = rhs.m_loBC;
  m_hiBC = rhs.m_hiBC;
  return *this;
}

PhysBCUtil::PhysBCUtil(const PhysBCUtil& rhs)
{
  m_loBC = rhs.m_loBC;
  m_hiBC = rhs.m_hiBC;
}


void
PhysBCUtil::setBCs()
{
  ParmParse ppBC("physBC");

  vector<int> tempBC(SpaceDim);

  ppBC.getarr("lo", tempBC, 0, SpaceDim);
  for (int dir=0; dir<SpaceDim; dir++) {
    m_loBC[dir] = tempBC[dir];
  }

  ppBC.getarr("hi", tempBC, 0, SpaceDim);

  for (int dir=0; dir<SpaceDim; dir++) {
    m_hiBC[dir] = tempBC[dir];
  }

}


PhysBCUtil*
PhysBCUtil::newPhysBCUtil() const
{
  PhysBCUtil* newBCPtr = new PhysBCUtil;
  return newBCPtr;
}




// ----------------------------------------------------------
// basic utility functions
void 
PhysBCUtil::Dx(const Real a_dx) 
{
  m_dx = a_dx;
}


Real 
PhysBCUtil::Dx() const
{ 
  return m_dx;
}

void
PhysBCUtil::Time(const Real a_time) 
{
  m_time = a_time;
}

Real
PhysBCUtil::Time() const
{
  return m_time;
}


 
// -----------------------------------------------------------
// Now begin basic BC functions.  start with pressure BC's

// ---------------------------------------------------
DomainGhostBC*
PhysBCUtil::MacPressureBC() const
{
  DomainGhostBC* newBCPtr = basicPressureBC(false);
  return newBCPtr;
}



    
// ------------------------------------------------------
DomainGhostBC*
PhysBCUtil::LevelPressureBC() const
{
  DomainGhostBC* newBCPtr = basicPressureBC(false);
  return newBCPtr;
}


// ------------------------------------------------------
DomainGhostBC*
PhysBCUtil::SyncProjBC() const
{
  DomainGhostBC* newBCPtr = basicPressureBC(true);
  return newBCPtr;
}


// ------------------------------------------------------
DomainGhostBC*
PhysBCUtil::FreestreamCorrBC() const
{
  DomainGhostBC* newBCPtr = new DomainGhostBC;

  // this is different from all the other pressure BCs
  // in that it's the homogeneous form of the normal
  // velocity component BC

  SideIterator sit;
  for (int dir=0; dir<SpaceDim; dir++) {
    for (sit.reset(); sit.ok(); ++sit) {
      int bctype = getBC(dir,sit());
      if (bctype == Inflow) {
	NeumannBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == Outflow) {
	NeumannBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == SolidWall) {
	NeumannBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == noShear) {
	NeumannBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else {
	MayDay::Error("FreestreamCorrBC -- bad BC type");
      } 
    }

  }

  return newBCPtr;
}




// ------------------------------------------------------
DomainGhostBC*
PhysBCUtil::InitialProjBC() const
{

  // initial projection BC should be the same as normal sync projection
  DomainGhostBC* newBCPtr = basicPressureBC(true);
  return newBCPtr;

}


// now do pressure gradient BC's.


// ------------------------------------------------------
DomainGhostBC*
PhysBCUtil::gradMacPressureBC() const
{
  DomainGhostBC* newBCPtr = basicGradPressureBC();
  return newBCPtr;
}


// ------------------------------------------------------
DomainGhostBC*
PhysBCUtil::gradPiBC() const 
{
  DomainGhostBC* newBCPtr = basicGradPressureBC();
  return newBCPtr;
}


// ------------------------------------------------------
DomainGhostBC*
PhysBCUtil::gradESyncBC() const 
{
  DomainGhostBC* newBCPtr = basicGradPressureBC();
  return newBCPtr;
}

// ------------------------------------------------------
DomainGhostBC*
PhysBCUtil::gradELambdaBC() const 
{
  DomainGhostBC* newBCPtr = basicGradPressureBC();
  return newBCPtr;
}



// ------------------------------------------------------
DomainGhostBC*
PhysBCUtil::gradInitialProjCorrBC() const 
{
  DomainGhostBC* newBCPtr = basicGradPressureBC();
  return newBCPtr;
}



// now do BC's on velocity
basicVelBC*
PhysBCUtil::viscousVelBC() const
{
  bool isHomogeneous = false;
  bool isViscous = true;
  basicVelBC* newBCPtr = basicCCVelBC(isHomogeneous, isViscous);
  return newBCPtr;
}


basicVelBC*
PhysBCUtil::tracingVelBC() const
{
  bool isHomogeneous = false;
  bool isViscous = false;
  basicVelBC* newBCPtr = basicCCVelBC(isHomogeneous, isViscous);
  return newBCPtr;
}


basicEdgeVelBC*
PhysBCUtil::advectionVelBC() const
{
  bool isHomogeneous = false;
  bool isViscous=true;
  bool isSingleComponent = true;
  basicEdgeVelBC* newBCPtr = basicECVelBC(isHomogeneous, isViscous, 
					  isSingleComponent);

  return newBCPtr;
}


basicEdgeVelBC*
PhysBCUtil::edgeVelBC() const
{
  bool isHomogeneous = false;
  bool isViscous = true;
  bool isSingleComponent = false;
  basicEdgeVelBC* newBCPtr = basicECVelBC(isHomogeneous, isViscous,
					  isSingleComponent);
  
  return newBCPtr;
}

basicVelBC*
PhysBCUtil::uDelUBC(bool isViscous) const
{
  bool isHomogeneous = true;
  basicVelBC* newBCPtr = basicCCVelBC(isHomogeneous, isViscous);
  return newBCPtr;

}

DomainGhostBC*
PhysBCUtil::viscousSolveBC(int a_dir) const
{
  bool isViscous = true;
  bool isHomogeneous = false;
  // assumption is that viscous Solves always deal with same interval
  Interval bcInterval(0,0);
  DomainGhostBC* newBCPtr = new DomainGhostBC;
  basicCCVelBC(*newBCPtr, isHomogeneous,isViscous,
	       a_dir, bcInterval);
  return newBCPtr;
}


basicVelBC* 
PhysBCUtil::uStarBC(bool a_isViscous) const
{
  bool isHomogeneous = false;
  basicVelBC* newBCPtr = basicCCVelBC(isHomogeneous, a_isViscous);
  return newBCPtr;
}


DomainGhostBC*
PhysBCUtil::viscousRefluxBC(int a_dir) const
{
  bool isViscous = true;
  bool isHomogeneous = true;
  // same assumption here as for the level-based viscous solves
  Interval bcInterval(0,0);
  DomainGhostBC* newBCPtr = new DomainGhostBC;
  basicCCVelBC(*newBCPtr,isHomogeneous,isViscous,
	       a_dir, bcInterval);
  return newBCPtr;
}


// ------------------------------------------------------
// now do scalar bc's 
// BC's on diffusion are different than those for tracing
// due to need to prevent diffusion from walls (except
// for things like temperature)
DomainGhostBC*
PhysBCUtil::diffusiveBC(int a_scalarType) const
{
  bool isHomogeneous = false;
  
  // this will probably be overridden by a problem-dependent 
  // function.  as it stands, this function does generic scalar BCs
  
  DomainGhostBC* newBCPtr = new DomainGhostBC;
  
  for (int dir=0; dir<SpaceDim; dir++) {
    SideIterator sit;
    for (sit.reset(); sit.ok(); ++sit) {
      int bctype = getBC(dir,sit());

      if (bctype == Inflow) {
	scalarInflowBC thisBC(dir,sit());
	thisBC.setScalarType(a_scalarType);
	thisBC.isHomogeneous(isHomogeneous);
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == Outflow) {
	ExtrapBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == SolidWall) {
	// homogeneous Neumann to prevent diffusive
	// flux through solid walls
	NeumannBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == noShear) {
	// no diffusive flux through boundary
	NeumannBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else {
	MayDay::Error("PhysBCUtil::diffusiveBC -- invalid bctype");
      }

    } // end loop over sides
  } // end iteration over directions


  return newBCPtr;
}

// -------------------------------------------------------
// 
DomainGhostBC*
PhysBCUtil::scalarTraceBC(int a_scalarType) const
{
  bool isHomogeneous = false;
  DomainGhostBC* newBCPtr = basicScalBC(isHomogeneous,
					a_scalarType);
  return newBCPtr;
}


// -------------------------------------------------------
DomainGhostBC*
PhysBCUtil::scalarDiffusionSolveBC(int a_scalarType) const
{
  bool isHomogeneous = false;
  DomainGhostBC* newBCPtr = basicScalBC(isHomogeneous,
					a_scalarType);

  return newBCPtr;
}


// -------------------------------------------------------
DomainGhostBC* 
PhysBCUtil::scalarRefluxSolveBC(int a_scalarType) const
{
  bool isHomogeneous = true;
  DomainGhostBC* newBCPtr = basicScalBC(isHomogeneous,
					a_scalarType);
  return newBCPtr;

}


// --------------------------------------------------------
DomainGhostBC*
PhysBCUtil::lambdaBC() const
{
  DomainGhostBC* newBCPtr = new DomainGhostBC;

  for (int dir=0; dir<SpaceDim; dir++) {
    SideIterator sit;
    for (sit.reset(); sit.ok(); ++sit) {
      int bctype = getBC(dir,sit());
      if (bctype == SolidWall) {
	ExtrapBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == Inflow) {
	scalarDirichletBC thisBC(dir,sit());
	thisBC.setVal(1.0);
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == Outflow) {
	ExtrapBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == noShear) {
	ExtrapBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else {
	MayDay::Error("PhysBCUtil::lambdaBC -- bad bctype");
      } 

    } // end iteration over lo-hi
  } // end iteration over directions

  return newBCPtr;
}


// -------------------------------------------------------
basicVelBC* 
PhysBCUtil::viscousSrcBC() const
{
  // this is just an extrapolation BC to make sure that things
  // have reasonable values, esp. in corner cells.

  viscousBC* newBCPtr = new viscousBC;
  return static_cast<basicVelBC*>(newBCPtr);
}


// -------------------------------------------------------
basicVelBC* 
PhysBCUtil::diffusiveSrcBC(int scalarType) const
{
  // note that this returns the same BC as the viscousSrcBC
  basicVelBC* newBCPtr = new viscousBC;
  return newBCPtr;
}


DomainGhostBC*
PhysBCUtil::streamFunctionBC(const Interval& a_comps) const
{
  DomainGhostBC* newBCPtr = new DomainGhostBC;

  for (int dir=0; dir<SpaceDim; dir++) {
    SideIterator sit;
    for (sit.reset(); sit.ok(); ++sit) {
      int bctype = getBC(dir,sit());
      if (bctype == SolidWall) {
	DircheletBC thisBC(dir,sit(),a_comps);
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == Inflow) {
	MayDay::Error("Don't know how to set stream function inflow");
	scalarDirichletBC thisBC(dir,sit(),a_comps);
	thisBC.setVal(1.0);
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == Outflow) {
	ExtrapBC thisBC(dir,sit(),a_comps);
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == noShear) {
	DircheletBC thisBC(dir,sit(),a_comps);
	newBCPtr->setBoxGhostBC(thisBC);
      } else {
	MayDay::Error("PhysBCUtil::streamFunctionBC -- bad bctype");
      } 

    } // end iteration over lo-hi
  } // end iteration over directions

  return newBCPtr;
}



  
// -------------------------------------------------------
// these functions do all the real work...
DomainGhostBC*
PhysBCUtil::basicPressureBC(bool a_isHomogeneous) const
{
  
  DomainGhostBC* newBCPtr = new DomainGhostBC;
  
  // loop over directions
  for (int dir=0; dir<SpaceDim; dir++) {
    SideIterator sit;
    for (sit.reset(); sit.ok(); ++sit) {
      int bctype = getBC(dir,sit());
      if (bctype == SolidWall) {
	NeumannBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == Inflow) {
	NeumannBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == Outflow) {
	DircheletBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == noShear) {
	NeumannBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == Symmetry) {
        ReflectBC thisBC(dir,sit());
        newBCPtr->setBoxGhostBC(thisBC);
      } else {
	MayDay::Error("PhysBCUtil::basicPressureBC -- bad bctype");
      }
    } 
  }
  
  return newBCPtr;

}  


DomainGhostBC*
PhysBCUtil::basicGradPressureBC() const
{
  DomainGhostBC* newBCPtr = new DomainGhostBC;
  
  // loop over directions
  for (int dir=0; dir<SpaceDim; ++dir) {
    SideIterator sit;
    for (sit.reset(); sit.ok(); ++sit) {
      int bctype = getBC(dir,sit());
      if (bctype == SolidWall) {
	HOExtrapBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == Inflow) {
	HOExtrapBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == Outflow) {
	HOExtrapBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == noShear) {
	HOExtrapBC thisBC(dir,sit());
	//DircheletBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else {
	MayDay::Error("PhysBCUtil::basicGradPressureBC -- bad bctype");
      }
    } // end loop over sides
  } // end loop over directions
  
  return newBCPtr;
  
}


basicVelBC*
PhysBCUtil::basicCCVelBC(bool a_isHomogeneous, bool a_isViscous) const
{

  basicVelBC* newBCPtr = new basicVelBC;

  // loop over velocity components
  for (int dir=0; dir<SpaceDim; dir++) {
    DomainGhostBC thisCompBC;
    Interval compInterval(dir,dir);

    basicCCVelBC(thisCompBC, a_isHomogeneous,
		 a_isViscous, dir, compInterval);

    newBCPtr->setBCType(thisCompBC, dir);
  }

  return newBCPtr;
}


void
PhysBCUtil::basicCCVelBC(DomainGhostBC& a_newBC,
			 bool a_isHomogeneous,
			 bool a_isViscous, 
			 int a_comp, Interval& a_interval) const
{

  // loop over directions
  for (int dir=0; dir<SpaceDim; dir++) {
    // loop over hi-lo
    SideIterator sit;
    for (sit.begin(); sit.ok(); ++sit) {
      
      int bctype = getBC(dir,sit());
      
      if (bctype == SolidWall) {
	// normal velocity BC's are always no-flow
	if (dir == a_comp) {
	  DircheletBC thisBC(dir,sit(), a_interval);
	  a_newBC.setBoxGhostBC(thisBC);
	} else {
	  // tangential BCs are no-slip if viscous, extrap if invisice
	  if (a_isViscous) {
	    DircheletBC thisBC(dir,sit(), a_interval);
	    a_newBC.setBoxGhostBC(thisBC);
	  } else {
	    ExtrapBC thisBC(dir,sit(),a_interval);
	    a_newBC.setBoxGhostBC(thisBC);
	  }

	} // end if tangential
      } else if (bctype == Inflow) {
	// this will most likely get overwritten in a derived class
	InflowVelBC thisBC(dir,sit(),a_interval);
	thisBC.isHomogeneous(a_isHomogeneous);
	thisBC.setBCVal(1.0);
	a_newBC.setBoxGhostBC(thisBC);
      } else if (bctype == Outflow) {
	NeumannBC thisBC(dir,sit(), a_interval);
	a_newBC.setBoxGhostBC(thisBC);
      } else if (bctype == noShear) {
	// normal velocity BC's still no-flow
	if (dir == a_comp) {
	  DircheletBC thisBC(dir,sit(), a_interval);
	  a_newBC.setBoxGhostBC(thisBC);
	} else {
	  // tangential BC is no-shear
	  NeumannBC thisBC(dir,sit(),a_interval);
	  a_newBC.setBoxGhostBC(thisBC);
	} // end if tangential
      } else {
	MayDay::Error("PhysBCUtil::basicCCVelBC -- bad bctype");
      }

    } // end iteration over sides
  } // end iteration over directions
}
	
	
	    
	  
DomainGhostBC*
PhysBCUtil::basicScalBC(bool a_isHomogeneous,
			int a_scalType) const
{

  DomainGhostBC* newBCPtr = new DomainGhostBC;

  // this should probably be overridden by 
  // a problem-dependent function.  As it 
  // stands, this function does generic 
  // scalar BCs

  for (int dir=0; dir<SpaceDim; dir++) {
    SideIterator sit;
    for (sit.reset(); sit.ok(); ++sit) {
      int bctype = getBC(dir,sit());
      if (bctype == Inflow) {
	MayDay::Error("Should not be in PhysBCUtil::basicScalBC");
	scalarInflowBC thisBC(dir,sit());
	thisBC.setScalarType(a_scalType);
	thisBC.isHomogeneous(a_isHomogeneous);
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == Outflow) {
	ExtrapBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == SolidWall) {
	ExtrapBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else if (bctype == noShear) {
	ExtrapBC thisBC(dir,sit());
	newBCPtr->setBoxGhostBC(thisBC);
      } else {
	MayDay::Error("PhysBCUtil::basicScalBC -- invalid bctype");
      }

    } // end loop over sides
  } // end iteration over directions

  return newBCPtr;

}
  

basicEdgeVelBC*
PhysBCUtil::basicECVelBC(bool a_isHomogeneous, bool a_isViscous,
			 bool a_isSingleComponent) const
{

  basicEdgeVelBC* newBCPtr = new basicEdgeVelBC;
  // at the moment, only single component defined
  assert (a_isSingleComponent == true);
  newBCPtr->isSingleComponent(a_isSingleComponent);

  // loop over component directions
  for (int comp=0; comp<SpaceDim; comp++) {
    DomainGhostBC thisCompBC;
    Interval compInterval(0,0);

    IndexType edgeVelType;
    // first set all directions to be cell-centered
    edgeVelType.clear();
    // then set normal direction to be edge-centered
    edgeVelType.set(comp);
    
    // loop over directions
    for (int dir=0; dir<SpaceDim; dir++) {
      SideIterator sit;
      for (sit.begin(); sit.ok(); ++sit) {
	int bctype = getBC(dir,sit());
	
	// solid-wall BCs
	if (bctype == SolidWall) {
	  // normal BC's are always no-flow
	  if (dir == comp) {
	    edgeDirichletBC thisBC(dir,sit(), compInterval);
	    thisBC.ixType(edgeVelType);
	    thisBC.setBCVal(0.0);
	    thisCompBC.setBoxGhostBC(thisBC);
	  } else {
	    // tangential BC's are no-slip if viscous, extrap if inviscid
	    if (a_isViscous) {
	      DircheletBC thisBC(dir,sit(),compInterval);
	      thisCompBC.setBoxGhostBC(thisBC);
	    } else {
	      ExtrapBC thisBC(dir,sit(),compInterval);
	      thisCompBC.setBoxGhostBC(thisBC);
	    }
	  } // end if tangential comp
	} else if (bctype == Inflow) {
	  edgeDirichletBC thisBC(dir,sit(),compInterval);
	  thisBC.ixType(edgeVelType);
	  if (dir == comp) {
	    // normal velocity is uniform
	    thisBC.setBCVal(1.0);
	  } else {
	    // no tangential velocity at inflow
	    thisBC.setBCVal(0.0);
	  }
	  thisCompBC.setBoxGhostBC(thisBC);
	} else if (bctype == Outflow) {
	  // this is set to whatever it is set to by MAC
	  NoOpBC thisBC(dir,sit(),compInterval);
	  thisCompBC.setBoxGhostBC(thisBC);
	} else if (bctype == noShear) {
	  if (dir == comp) {
	    // normal velocity is no-flow
	    edgeDirichletBC thisBC(dir,sit(),compInterval);
	    thisBC.ixType(edgeVelType);
	    thisBC.setBCVal(0.0);
	    thisCompBC.setBoxGhostBC(thisBC);
	  } else {
	    // tangential BC is do-nothing BC???
	    NoOpBC thisBC(dir,sit(),compInterval);
	    thisCompBC.setBoxGhostBC(thisBC);
	  }
	} else {
	  MayDay::Error("PhysBCUtil::basicECVelBC -- bad bctype");
	}

      } // end loop over hi-lo
    } // end loop over directions

    newBCPtr->setBCType(thisCompBC,comp);
    
  } // end loop over velocity components

  return newBCPtr;
}


void
PhysBCUtil::computeBoundaryDt(Real& a_dt, Real a_cfl, Real a_dx) const 
{
  // in the default case, nothing is done here.
}
