/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/

// basicVelBC.H

//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// Dan Martin, Fri, Jan 14, 2000

#ifndef _basicVelBC_H_
#define _basicVelBC_H_

#include "FArrayBox.H"
#include "FluxBox.H"
#include "REAL.H"
#include "LevelData.H"
#include "GhostBC.H"
#include "ProblemDomain.H"


/// this is a basic physical BC class designed to handle velocities
class basicVelBC
/** this class is handles velocities, which are by their nature 
    multicomponent */
{
public:
  
  ///
  basicVelBC();
  
  /// 
  virtual ~basicVelBC();

  /// one DomainGhostBC is set for each velocity component 
  virtual void setBCType(const DomainGhostBC& a_ghostBC, int a_component);


  ///
  basicVelBC(const int a_component);
  
  /// 
  virtual void applyBCs(FArrayBox& a_state, const ProblemDomain& a_domain,
			const Real a_dx) const;

  /// old interface using Box instead of ProblemDomain
  virtual void applyBCs(FArrayBox& a_state, const Box& a_domain,
			const Real a_dx) const;

  /// 
  virtual void applyBCs(LevelData<FArrayBox>& a_stateMF, 
			const ProblemDomain& a_domain, 
                        const Real a_dx) const;

  ///  old interface using Box instead of ProblemDomain
  virtual void applyBCs(LevelData<FArrayBox>& a_stateMF, 
			const Box& a_domain, const Real a_dx) const;

  /// single-component version
  virtual void applyBCs(FArrayBox& a_state, 
                        const ProblemDomain& a_domain,
			const Real a_dx, int a_comp) const;

  /// single-component version using Box instead of ProblemDomain
  virtual void applyBCs(FArrayBox& a_state, const Box& a_domain,
			const Real a_dx, int a_comp) const;

  /// returns DomainGhostBC for direction Dir
  const DomainGhostBC& DomGhostBC(const int dir) const;

protected:
  ///
  Tuple<DomainGhostBC*, SpaceDim> m_componentBC;


private:

};



/// this is a basic physical BC class designed to handle velocities
class basicEdgeVelBC
/** this class is handles velocities, which are by their nature 
    multicomponent */
{
public:
  
  ///
  basicEdgeVelBC();
  
  /// 
  virtual ~basicEdgeVelBC();

  /// one DomainGhostBC is set for each velocity component 
  virtual void setBCType(const DomainGhostBC& a_ghostBC, int a_component);


  ///
  basicEdgeVelBC(const int a_component);
  
  /// 
  virtual void applyBCs(FluxBox& a_state, 
                        const ProblemDomain& a_domain,
			const Real a_dx) const;

  /// old interface using Box instead of ProblemDomain
  virtual void applyBCs(FluxBox& a_state, const Box& a_domain,
			const Real a_dx) const;

  /// 
  virtual void applyBCs(FArrayBox& a_state, const ProblemDomain& a_domain,
			const Real a_dx) const;

  /// old interface using Box instead of ProblemDomain
  virtual void applyBCs(FArrayBox& a_state, const Box& a_domain,
			const Real a_dx) const;


  /// 
  virtual void applyBCs(LevelData<FluxBox>& a_stateMF, 
			const ProblemDomain& a_domain, 
                        const Real a_dx) const;


  /// old interface using Box instead of ProblemDomain
  virtual void applyBCs(LevelData<FluxBox>& a_stateMF, 
			const Box& a_domain, const Real a_dx) const;
  

  /// single-component version
  virtual void applyBCs(FluxBox& a_state, 
                        const ProblemDomain& a_domain,
			const Real a_dx, int a_comp) const;

  /// single-component version using Box instead of ProblemDomain
  virtual void applyBCs(FluxBox& a_state, const Box& a_domain,
			const Real a_dx, int a_comp) const;


  /// single-component version
  virtual void applyBCs(FArrayBox& a_state, 
                        const ProblemDomain& a_domain,
			const Real a_dx, int a_comp) const;


  /// single-component version using Box instead of ProblemDomain
  virtual void applyBCs(FArrayBox& a_state, const Box& a_domain,
			const Real a_dx, int a_comp) const;


  /// returns DomainGhostBC for direction Dir
  const DomainGhostBC& DomGhostBC(const int dir) const;

  /// will this be applied to single-component FluxBoxes?
  /** Single component Fluxboxes, like advection velocities,
      will have each direction be a single BC (x-dir fab is
      x-velocity, y-dir fab is y-velocity, etc).  multicomponent
      edge-centered velocities are arranged so that velocity
      components align with components in Fabs, so that the
      x-edge-centered fab will have SpaceDim components:
      x-velocity, y-velocity, (and z-velocity in 3d).  Each
      of these will have a different BC.
  */
  void isSingleComponent(bool a_isSingleComp);

  /// will this be applied to single-component FluxBoxes?
  bool isSingleComponent() const;

protected:
  ///
  Tuple<DomainGhostBC*, SpaceDim> m_componentBC;

  /// if this is will be applied to single component FluxBoxes
  bool m_isSingleComponent;

private:

};


/// does inflow
class InflowVelBC: public BoxGhostBC
{
public:
  friend class DomainGhostBC;

  /// 
  InflowVelBC();

  /// 
  virtual ~InflowVelBC();

  ///
  InflowVelBC(int dir, Side::LoHiSide sd);
  
  ///
  InflowVelBC(int dir, Side::LoHiSide sd,
	      const Interval& a_comps);


  /// is the BC homogeneous?
  void isHomogeneous(bool a_isHomogeneous);
  
  ///
  bool isHomogeneous() const;

  /// for single-valued inflow...
  void setBCVal(const Real a_bcVal);
  
  ///
  Real BCVal() const;
  

  /// set time (for time-varying inflow
  void Time(const Real m_time);

  ///
  Real Time() const;

protected:

  ///
  virtual void fillBCValues(FArrayBox& a_neumfac,
			    FArrayBox& a_dirfac,
			    FArrayBox& a_inhmval,
			    Real a_dx,
			    const ProblemDomain& a_domain) const;


  /// deprecated interface
  virtual void fillBCValues(FArrayBox& a_neumfac,
			    FArrayBox& a_dirfac,
			    FArrayBox& a_inhmval,
			    Real a_dx,
			    const Box& a_domain) const;
  
  virtual BoxGhostBC* new_boxghostbc() const;


private:
  bool m_isHomogeneous;

  Real m_bcVal;

  Real m_time;

  InflowVelBC(const InflowVelBC& bcin) {;}
  void operator=(const InflowVelBC& bcin) {;}

};
  

#endif


